"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _typeorm = require("typeorm");

var _crypto = require("crypto");

var _errors = require("../../lib/errors");

var _config = _interopRequireDefault(require("./lib/config"));

var _transform = _interopRequireDefault(require("./lib/transform"));

var _models = _interopRequireDefault(require("./models"));

var _utils = require("./lib/utils");

var _requireOptional = _interopRequireDefault(require("@balazsorban/require-optional"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const Adapter = (typeOrmConfig, options = {}) => {
  const typeOrmConfigObject = typeof typeOrmConfig === 'string' ? _config.default.parseConnectionString(typeOrmConfig) : typeOrmConfig;
  const {
    models: customModels = {}
  } = options;
  const models = {
    User: customModels.User ? customModels.User : _models.default.User,
    Account: customModels.Account ? customModels.Account : _models.default.Account,
    Session: customModels.Session ? customModels.Session : _models.default.Session,
    VerificationRequest: customModels.VerificationRequest ? customModels.VerificationRequest : _models.default.VerificationRequest
  };
  (0, _transform.default)(typeOrmConfigObject, models, options);

  const config = _config.default.loadConfig(typeOrmConfigObject, { ...options,
    models
  });

  const User = models.User.model;
  const Account = models.Account.model;
  const Session = models.Session.model;
  const VerificationRequest = models.VerificationRequest.model;
  let connection = null;

  async function getAdapter(appOptions) {
    const {
      logger
    } = appOptions;

    function debug(debugCode, ...args) {
      logger.debug(`TYPEORM_${debugCode}`, ...args);
    }

    async function _connect() {
      connection = (0, _typeorm.getConnection)(config.name);

      if (!connection.isConnected) {
        connection = await connection.connect();
      }
    }

    if (!connection) {
      try {
        connection = await (0, _typeorm.createConnection)(config);
      } catch (error) {
        if (error.name === 'AlreadyHasActiveConnectionError') {
          await _connect();
        } else {
          logger.error('ADAPTER_CONNECTION_ERROR', error);
        }
      }
    } else {
      await _connect();
    }

    if (process.env.NODE_ENV !== 'production') {
      await (0, _utils.updateConnectionEntities)(connection, config.entities);
    }

    const {
      manager
    } = connection;
    let idKey = 'id';
    let ObjectId;

    if (config.type === 'mongodb') {
      idKey = '_id';
      const mongodb = (0, _requireOptional.default)('mongodb');
      ObjectId = mongodb.ObjectID;
    }

    if (appOptions && (!appOptions.session || !appOptions.session.maxAge)) {
      debug('GET_ADAPTER', 'Session expiry not configured (defaulting to 30 days');
    }

    const defaultSessionMaxAge = 30 * 24 * 60 * 60 * 1000;
    const sessionMaxAge = appOptions && appOptions.session && appOptions.session.maxAge ? appOptions.session.maxAge * 1000 : defaultSessionMaxAge;
    const sessionUpdateAge = appOptions && appOptions.session && appOptions.session.updateAge ? appOptions.session.updateAge * 1000 : 0;

    async function createUser(profile) {
      debug('CREATE_USER', profile);

      try {
        const user = new User(profile.name, profile.email, profile.image, profile.emailVerified);
        return await manager.save(user);
      } catch (error) {
        logger.error('CREATE_USER_ERROR', error);
        return Promise.reject(new _errors.CreateUserError(error));
      }
    }

    async function getUser(id) {
      debug('GET_USER', id);

      if (ObjectId && !(id instanceof ObjectId)) {
        id = ObjectId(id);
      }

      try {
        return manager.findOne(User, {
          [idKey]: id
        });
      } catch (error) {
        logger.error('GET_USER_BY_ID_ERROR', error);
        return Promise.reject(new Error('GET_USER_BY_ID_ERROR', error));
      }
    }

    async function getUserByEmail(email) {
      debug('GET_USER_BY_EMAIL', email);

      try {
        if (!email) {
          return Promise.resolve(null);
        }

        return manager.findOne(User, {
          email
        });
      } catch (error) {
        logger.error('GET_USER_BY_EMAIL_ERROR', error);
        return Promise.reject(new Error('GET_USER_BY_EMAIL_ERROR', error));
      }
    }

    async function getUserByProviderAccountId(providerId, providerAccountId) {
      debug('GET_USER_BY_PROVIDER_ACCOUNT_ID', providerId, providerAccountId);

      try {
        const account = await manager.findOne(Account, {
          providerId,
          providerAccountId
        });

        if (!account) {
          return null;
        }

        return manager.findOne(User, {
          [idKey]: account.userId
        });
      } catch (error) {
        logger.error('GET_USER_BY_PROVIDER_ACCOUNT_ID_ERROR', error);
        return Promise.reject(new Error('GET_USER_BY_PROVIDER_ACCOUNT_ID_ERROR', error));
      }
    }

    async function updateUser(user) {
      debug('UPDATE_USER', user);
      return manager.save(User, user);
    }

    async function deleteUser(userId) {
      debug('DELETE_USER', userId);
      return false;
    }

    async function linkAccount(userId, providerId, providerType, providerAccountId, refreshToken, accessToken, accessTokenExpires) {
      debug('LINK_ACCOUNT', userId, providerId, providerType, providerAccountId, refreshToken, accessToken, accessTokenExpires);

      try {
        const account = new Account(userId, providerId, providerType, providerAccountId, refreshToken, accessToken, accessTokenExpires);
        return manager.save(account);
      } catch (error) {
        logger.error('LINK_ACCOUNT_ERROR', error);
        return Promise.reject(new Error('LINK_ACCOUNT_ERROR', error));
      }
    }

    async function unlinkAccount(userId, providerId, providerAccountId) {
      debug('UNLINK_ACCOUNT', userId, providerId, providerAccountId);
      return false;
    }

    async function createSession(user) {
      debug('CREATE_SESSION', user);

      try {
        let expires = null;

        if (sessionMaxAge) {
          const dateExpires = new Date();
          dateExpires.setTime(dateExpires.getTime() + sessionMaxAge);
          expires = dateExpires;
        }

        const session = new Session(user.id, expires);
        return manager.save(session);
      } catch (error) {
        logger.error('CREATE_SESSION_ERROR', error);
        return Promise.reject(new Error('CREATE_SESSION_ERROR', error));
      }
    }

    async function getSession(sessionToken) {
      debug('GET_SESSION', sessionToken);

      try {
        const session = await manager.findOne(Session, {
          sessionToken
        });

        if (session && session.expires && new Date() > new Date(session.expires)) {
          return null;
        }

        return session;
      } catch (error) {
        logger.error('GET_SESSION_ERROR', error);
        return Promise.reject(new Error('GET_SESSION_ERROR', error));
      }
    }

    async function updateSession(session, force) {
      debug('UPDATE_SESSION', session);

      try {
        if (sessionMaxAge && (sessionUpdateAge || sessionUpdateAge === 0) && session.expires) {
          const dateSessionIsDueToBeUpdated = new Date(session.expires);
          dateSessionIsDueToBeUpdated.setTime(dateSessionIsDueToBeUpdated.getTime() - sessionMaxAge);
          dateSessionIsDueToBeUpdated.setTime(dateSessionIsDueToBeUpdated.getTime() + sessionUpdateAge);

          if (new Date() > dateSessionIsDueToBeUpdated) {
            const newExpiryDate = new Date();
            newExpiryDate.setTime(newExpiryDate.getTime() + sessionMaxAge);
            session.expires = newExpiryDate;
          } else if (!force) {
            return null;
          }
        } else {
          if (!force) {
            return null;
          }
        }

        return manager.save(Session, session);
      } catch (error) {
        logger.error('UPDATE_SESSION_ERROR', error);
        return Promise.reject(new Error('UPDATE_SESSION_ERROR', error));
      }
    }

    async function deleteSession(sessionToken) {
      debug('DELETE_SESSION', sessionToken);

      try {
        return await manager.delete(Session, {
          sessionToken
        });
      } catch (error) {
        logger.error('DELETE_SESSION_ERROR', error);
        return Promise.reject(new Error('DELETE_SESSION_ERROR', error));
      }
    }

    async function createVerificationRequest(identifier, url, token, secret, provider) {
      debug('CREATE_VERIFICATION_REQUEST', identifier);

      try {
        const {
          baseUrl
        } = appOptions;
        const {
          sendVerificationRequest,
          maxAge
        } = provider;
        const hashedToken = (0, _crypto.createHash)('sha256').update(`${token}${secret}`).digest('hex');
        let expires = null;

        if (maxAge) {
          const dateExpires = new Date();
          dateExpires.setTime(dateExpires.getTime() + maxAge * 1000);
          expires = dateExpires;
        }

        const newVerificationRequest = new VerificationRequest(identifier, hashedToken, expires);
        const verificationRequest = await manager.save(newVerificationRequest);
        await sendVerificationRequest({
          identifier,
          url,
          token,
          baseUrl,
          provider
        });
        return verificationRequest;
      } catch (error) {
        logger.error('CREATE_VERIFICATION_REQUEST_ERROR', error);
        return Promise.reject(new Error('CREATE_VERIFICATION_REQUEST_ERROR', error));
      }
    }

    async function getVerificationRequest(identifier, token, secret, provider) {
      debug('GET_VERIFICATION_REQUEST', identifier, token);

      try {
        const hashedToken = (0, _crypto.createHash)('sha256').update(`${token}${secret}`).digest('hex');
        const verificationRequest = await manager.findOne(VerificationRequest, {
          identifier,
          token: hashedToken
        });

        if (verificationRequest && verificationRequest.expires && new Date() > new Date(verificationRequest.expires)) {
          await manager.delete(VerificationRequest, {
            identifier,
            token: hashedToken
          });
          return null;
        }

        return verificationRequest;
      } catch (error) {
        logger.error('GET_VERIFICATION_REQUEST_ERROR', error);
        return Promise.reject(new Error('GET_VERIFICATION_REQUEST_ERROR', error));
      }
    }

    async function deleteVerificationRequest(identifier, token, secret, provider) {
      debug('DELETE_VERIFICATION', identifier, token);

      try {
        const hashedToken = (0, _crypto.createHash)('sha256').update(`${token}${secret}`).digest('hex');
        await manager.delete(VerificationRequest, {
          identifier,
          token: hashedToken
        });
      } catch (error) {
        logger.error('DELETE_VERIFICATION_REQUEST_ERROR', error);
        return Promise.reject(new Error('DELETE_VERIFICATION_REQUEST_ERROR', error));
      }
    }

    return Promise.resolve({
      createUser,
      getUser,
      getUserByEmail,
      getUserByProviderAccountId,
      updateUser,
      deleteUser,
      linkAccount,
      unlinkAccount,
      createSession,
      getSession,
      updateSession,
      deleteSession,
      createVerificationRequest,
      getVerificationRequest,
      deleteVerificationRequest
    });
  }

  return {
    getAdapter
  };
};

var _default = {
  Adapter,
  Models: _models.default
};
exports.default = _default;